/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 *
 * Author Peter Kunszt <peter.kunszt@cern.ch>
 * $Id: Transfer.hxx,v 1.27 2006/07/13 09:18:48 badino Exp $
 */

#ifndef GLITE_DATA_TRANSFER_HXX_
#define GLITE_DATA_TRANSFER_HXX_

#include "glite/data/transfer/TransferBase.hxx"
#include "glite/data/transfer/glite-gridftp-performance.h"

/*
 * Forward declarations
 */
struct globus_gass_copy_handle_s;
	
namespace glite{
namespace transfer{


class TransferArgs : public TransferArgsBase {
public:
    TransferArgs(std::string source,
		 std::string destination) : TransferArgsBase(),
	m_source(source),
	m_destination(destination),
	m_srmput_timeout(1800),
	m_srmget_timeout(1800),
	m_xfer_timeout(1800),
	m_xfermarker_timeout(120),
	m_srmputdone_timeout(60),
	m_srmgetdone_timeout(60)
    {}

    const std::string &   source()             {return m_source;}
    const std::string &   destination()        {return m_destination;}

    int                   srmput_timeout()     {return m_srmput_timeout;}
    int                   srmget_timeout()     {return m_srmget_timeout;}
    int                   xfer_timeout()       {return m_xfer_timeout;}
    int                   xfermarker_timeout() {return m_xfermarker_timeout;}
    int                   srmputdone_timeout() {return m_srmputdone_timeout;}
    int                   srmgetdone_timeout() {return m_srmgetdone_timeout;}

    void setSRMPutTimeout    (int srmput_timeout)     {m_srmput_timeout     = srmput_timeout;}
    void setSRMGetTimeout    (int srmget_timeout)     {m_srmget_timeout     = srmget_timeout;}
    void setXferTimeout      (int xfer_timeout)       {m_xfer_timeout       = xfer_timeout;}
    void setXferMarkerTimeout(int xfermarker_timeout) {m_xfermarker_timeout = xfermarker_timeout;}
    void setSRMPutDoneTimeout(int srmputdone_timeout) {m_srmputdone_timeout = srmputdone_timeout;}
    void setSRMGetDoneTimeout(int srmgetdone_timeout) {m_srmgetdone_timeout = srmgetdone_timeout;}


private:
    TransferArgs() {}
    std::string           m_source;
    std::string           m_destination;
    int                   m_srmput_timeout;
    int                   m_srmget_timeout;
    int                   m_xfer_timeout;
    int                   m_xfermarker_timeout;
    int                   m_srmputdone_timeout;
    int                   m_srmgetdone_timeout;
};


/**
 * The TransferCommands is the main class to interact with the transfer.
 */

class Transfer : public TransferBase {
public:

    /**
     * Client Transfer object based on an existing ID
     */
    Transfer(const std::string &id);

    /**
     * Destructor
     */
    ~Transfer();

    /**
     * Spawn a transfer, retrieving the ID. This will not construct an
     * object but fork a separate process unless block is set to true,
     * in which case it blocks until the transfer is done.
     */
    static std::string start(TransferArgs &args, 
			     int (*change_procname)(const char *) = NULL);


    /**
     * Abort transfer
     */
    void stop();

    /**
     * retrieve transfer status
     */
    TransferSocket::Status status(perf_t *progress = NULL);

    /*
     * retrieve current transfer URLs being used - useful for SRM
     * transfers where the TURL is retrieved dynamically from the SRM.
     * The allocation and deallocation of these strings is up to the
     * caller.  The maximal length that the source and destination
     * strings can accept is the third argument. This will throw an
     * UninitializedException if the strings are too short to hold the
     * return values.
     */
    void getTURLs(std::string & source, std::string & destination);

    static void set_handle_callback(struct globus_gass_copy_handle_s *handle, int handle_set); // callback to get gridFTP handle for signalling

protected:
    /**
     * Default Constructor disabled - have to get it via a known ID
     */
    Transfer() {};

private:
    static int setSRMDone(void **c, void *context);

    static void cleanup(const char *msg, bool block, bool fail, bool deletesocket = false);

    static void *monitor_transfer(void *arg);
    static void monitor_transfer_stop(pthread_t monitor_thread);

    static const char* translate_srm_turl(void *client, void *context);

    static const char * srmStatusStr();

    static void *socket_listen(void *arg);

    static void copy_termination_handler(int signum);

    static void runDaemon(TransferArgs & args,
			  TransferFiles & transfile,
			  TransferFiles::UrlType sourceType,
			  TransferFiles::UrlType destType,
			  std::string & endpoint_src,
			  std::string & endpoint_dest
			  );


    static int s_status;
    static bool s_block;
    static char s_listenerror[BUF_LEN];  // error message from listener thread
    static char s_srmstatus[BUF_LEN];
    static int s_listener_exited;    // flag indicating whether listener has exited and how
    static TransferLogger *s_logger;
    static std::string s_id;
    static std::string s_errmsg;
    static bool s_done;
    static bool s_runningGridFTP;   // if we're inside the gridFTP call
    static pthread_t s_theGridFTPthread;   // the thread running the gridFTP (so it may be signalled)
    static int s_handle_set; // whether the gridFTP copy handle is currently valid or not
    static struct globus_gass_copy_handle_s * s_copy_handle; // the copy handle (so it can be aborted)
    static pthread_t s_listener;

};

} // namespace transfer
} // namespace glite

#endif //GLITE_DATA_TRANSFER_HXX_
