/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 *
 * Author Peter Kunszt <peter.kunszt@cern.ch>
 * $Id: TransferBase.hxx,v 1.6 2006/07/13 09:18:48 badino Exp $
 */

#ifndef GLITE_DATA_TRANSFER_BASE_HXX_
#define GLITE_DATA_TRANSFER_BASE_HXX_

#include <string>
#include <pthread.h>

#include "glite/data/transfer/TransferDefs.h"
#include "glite/data/transfer/TransferSocket.hxx"
#include "glite/data/transfer/TransferLogger.hxx"

#ifdef HAVE_FTIME
#undef HAVE_FTIME
#endif


namespace glite{
namespace transfer{


class TransferArgsBase {
public:
    TransferArgsBase() :
	m_prefix(NULL),
	m_nstreams(1),
	m_tcp_blocksize(0),
	m_block(false),
	m_check(false),
	m_loglevel(TransferLogger::INFO),
	m_console_logging(false),
	m_http_timeout(40),
	m_test(0),
	m_src_endpoint(NULL),
	m_dst_endpoint(NULL)
    {}

    const char *          prefix() const          {return m_prefix;}

    int                   nstreams() const        {return m_nstreams;}
    int                   tcp_blocksize() const   {return m_tcp_blocksize;}

    bool                  block() const           {return m_block;}
    bool                  check() const           {return m_check;}

    TransferLogger::Level loglevel() const        {return m_loglevel;}

    bool                  console_logging() const {return m_console_logging;}
    int                   test() const            {return m_test;}
    int                   http_timeout() const    {return m_http_timeout;}
    const char *          src_endpoint() const    {return m_src_endpoint;}
    const char *          dest_endpoint() const   {return m_dst_endpoint;}

    void setPrefix           (const char *prefix)     {m_prefix             = prefix;}
    void setLogLevel(TransferLogger::Level loglevel)  {m_loglevel           = loglevel;}
    void setStreams          (int n_streams)          {m_nstreams           = n_streams;}
    void setTCPBlockSize     (int n_tcp_blocksize)    {m_tcp_blocksize      = n_tcp_blocksize;}
    void setBlock            (bool block)             {m_block              = block;}
    void setCheck            (bool check)             {m_check              = check;}
    void setConsoleLogging   (bool console_logging)   {m_console_logging    = console_logging;}
    void setTest             (int test)               {m_test               = test;}
    void setHttpTimeout      (int t)                  {m_http_timeout       = t;}
    void setSrcEndpoint      (const char * e)         {m_src_endpoint       = e;}
    void setDestEndpoint     (const char * e)         {m_dst_endpoint       = e;}


private:
    TransferLogger::Level m_loglevel;
    const char *          m_prefix;
    int                   m_nstreams;
    int                   m_tcp_blocksize;
    bool                  m_block;
    bool                  m_check;
    bool                  m_console_logging;
    int                   m_test;
    int                   m_http_timeout;
    const char *          m_src_endpoint;
    const char *          m_dst_endpoint;
};		


/**
 * The TransferCommands is the main class to interact with the transfer.
 */

class TransferBase {
public:

    typedef enum {
	SUCCESS    = 0,   // the transfer finished successfully, there was no error!
        SRM_SOURCE = 1,   // error while resolving the source SURL
        SRM_DEST   = 2,   // error while resolving the destination SURL
        TRANSFER   = 3,   // transfer itself failed or the filesize check failed
        SECURITY   = 4,   // access errors
        NETWORK    = 5,   // clear network failure (no route to host, etc)
        SYNTAX     = 6,   // syntax error where some of the arguments are wrong
        ABORTED    = 7,   // the user aborted the transfer (issuing stop or killing the process) or listener thread died
	SRM_DONE_SOURCE = 8, // the final SRM 'setDone' operation failed on the source
	SRM_DONE_DEST = 9,   // the final SRM 'setDone' operation failed on the destination
	N_ERRTYPE  = 10
    } ErrorType;

    /**
     * Client Transfer object based on an existing ID
     */
    TransferBase(const std::string &id);

    /**
     * Destructor
     */
    ~TransferBase();

    /**
     * retrieve the failure type and corresponding final message
     */
    ErrorType getError(std::string & final);
	
    static const char * getErrorTypeStr(ErrorType e);

    static int getErrorType(std::string err);

    /**
     * Retrieve number of active transfers (if the start was issued 
     * with a prefix, only those with the given prefix are counted)
     */
    static int count(const char *prefix = NULL);

    /**
    * If the client calls need logging somewhere, set the logger here
    */
    void setLogger(TransferLogger* logger) {m_logger=logger; m_havelogger=true;}

    /**
     * Check what kind of transfer this is - true on srmCopy, false on gsiftp
     */
    bool isSRMCopy() {return m_files.isSRMCopy();}

    TransferFiles &getFiles() {return m_files;}

protected:

    TransferBase() {};  // no default constructor callable

    /**
    * helper methods for the static start command
    */

    void logerror(const char *msg);

    /* member variables */
    TransferFiles      m_files;
    TransferLogger    *m_logger;
    bool m_havelogger;
    static const char *s_errTypeStr[N_ERRTYPE];


    /* static members */
    static void get_endpoint(const std::string &name, std::string &endpoint);
    static void monitor_transfer_stop(pthread_t monitor_thread);
    static void setErrstr(std::string &errstr, const char *e, const char *msg);


};

} // namespace transfer
} // namespace glite

#endif //GLITE_DATA_TRANSFER_BASE_HXX_
