/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 *
 * Author Peter Kunszt <peter.kunszt@cern.ch>
 * $Id: TransferFiles.hxx,v 1.13 2006/07/13 09:18:48 badino Exp $
 */

#ifndef GLITE_DATA_TRANSFER_FILES_HXX_
#define GLITE_DATA_TRANSFER_FILES_HXX_

#include <string>
#include <vector>
#include <utility>
extern "C" {
#include <unistd.h>
}


namespace glite{
namespace transfer{

typedef std::vector<std::pair<std::string,std::string> > TransferList;

	
/**
 * The TransferFiles helper class assembles the name of the
 * socket file from the ID number and has methods to retrieve the
 * names of the failed and completed directory.
 */

class TransferFiles {
public:
    typedef enum {
	GSIFTP  = 0,
	SRM     = 1,
	LOCAL   = 2,
	HTTP    = 3,
	FTP     = 4
    } UrlType;

    /**
     * Default Constructor/Destructor
     */
    TransferFiles();
    ~TransferFiles() {};

    /**
     * Construct with an ID
     */
    TransferFiles(const std::string & id);

   /**
    * Create a new file with an optional prefix
    */
    void newfile(const char *prefix = NULL);

   /**
    * Start the transfer: write the initial data to the file
    */
    void start(const std::string &source, const std::string &destination, pid_t pid) const;

   /**
    * Start the transfer: write the initial data to the file
    */
    void startSRM(const TransferList &l, std::string endpoint) const;

    /**
     * Retrieve ID
     */
    const std::string & getID() const {return m_id;}

    /**
     * Retrieve socket file name
     */
    const std::string & getSocketFile() const {return m_sockfile;}

    /**
     * Retrieve mmap file name
     */
    const std::string & getMemoryFile() const {return m_memfile;}

    /**
     * Retrieve log file name
     */
    const std::string & getLogFile() const {return m_logfile;}

    /**
     * Retrieve log directory
     */
    const std::string & getLogDir() const {return m_logdir;}

    /**
     * Retrieve name of failed directory
     */
    const std::string & getFailedDir() const {return m_faileddir;}

    /**
     * Retrieve name of completed directory
     */
    const std::string & getCompletedDir() const {return m_completeddir;}

    /**
     * Retrieve name of failed logfile
     */
    const std::string & getFailedLogFile() const {return m_failedfile;}

    /**
     * Retrieve name of completed logfile
     */
    const std::string & getCompletedLogFile() const {return m_completedfile;}

    /**
     * Cleanup with a success flag: Move transfer log to 'completed'
     * directory. Throws exception if file is not there and also not
     * already in the completed directory
     */
    void success() const;
    
    /**
     * Cleanup with a failure flag: Move transfer log to 'failed'
     * directory.  Throws exception if file is not there and also not
     * already in the failed directory
     */
    int failure();

    /**
     * Retrieve pid of process identified by the ID.
     */
    pid_t pid();

    /**
     * Check if the given ID has already completed
     */
    bool checkCompleted() const;

    /**
     * Check if the given ID has failed.
     */
    bool checkFailed() const;

    /**
     * Check if the given ID is currently active
     */
    bool checkActive();

    /** Check if the transfer with the given destination is already in progress
     * this will throw an AlreadyRunningException if there is such a transfer.
     */
    void checkTransfer (const std::string &dest, const char *prefix) const;

    /**
     * Count the currently active transfers with the given prefix.
     */
    int countActive (const char *prefix) const;

    /*
     * Retrieve the URL type. Throws InvalidURLException on unknown URLs.
     */
    static UrlType getURLType(const std::string &url);

    void killProcess();
    void writeLogLine(const std::string &line) const;

    /**
     * Retrieve final message
     */
    int getError(std::string & msg);

    /**
     * Retrieve actual transfer URLs
     */
    void getTURLs(std::string &source, std::string & dest);

    long long getSize() { return m_size; }
    void printTURL(const char *source, const char *dest);
    void writeFileSize(long long filesize) const;
    bool isSRMCopy() { pid();return m_srm; }
    void setSRMCopy() { m_srm = true; }
    void putSRMReqId(int id) const;
    int getSRMReqId();
    void getSRMEndpoint(std::string &endpoint);

    void *memmapfailed(size_t len, bool write);
    void *memmap(size_t& len, bool write);
    void memunmap(size_t len, void *m, bool deletefile);
    void memunmapfailed(size_t len, void *m);
	
protected:
    void addTransfer(char *source, char *dest, int pid);
    void removeTransfer();
    void checkTransfer(char *dest);
    int countTransfer();
    void init();
    void initID(const char *id, const char *prefix, bool newfile);
    void setSize(long long size) { m_size = size; }
    void *doMemmap(size_t& len, bool write, const std::string &fname);

private:
    std::string m_id;
    std::string m_sockfile;
    std::string m_memfile;
    std::string m_memfilefailed;
    std::string m_logfile;
    std::string m_logdir;
    std::string m_faileddir;
    std::string m_completeddir;
    std::string m_failedfile;
    std::string m_completedfile;
    std::string m_listfile;
    std::string m_listfilelock;
    std::string m_srm_endpoint;
    long long m_size;
    pid_t m_pid;
    bool m_srm;
    int m_srm_size;

    friend class Transfer;
};

} // namespace transfer
} // namespace glite

#endif //GLITE_DATA_TRANSFER_FILES_HXX_
