/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 *
 * Author Paolo Badino <peter.kunszt@cern.ch>
 * $Id: TransferSRM.hxx,v 1.10 2006/07/13 15:43:39 badino Exp $
 */

#ifndef GLITE_DATA_TRANSFER_SRM_HXX_
#define GLITE_DATA_TRANSFER_SRM_HXX_

#include "glite/data/transfer/TransferBase.hxx"
#include "glite/data/transfer/TransferFiles.hxx"
#include <vector>

#ifdef HAVE_FTIME
#undef HAVE_FTIME
#endif

/*
 * Forward Declaration
 */
namespace srm{
    class srm2__RequestStatus;
    class SrmUtil;
}

namespace glite{
namespace transfer{

/**
 * TransferSRMFileStat Structure (Public)
 */	
struct TransferSRMFileStat {
    int     fileId;
    int     status;
	off64_t size;
    char    message[2048];
    
    /**
     * Constructor
     */
    TransferSRMFileStat():fileId(-1),status(-1),size(0){
        memset(message,0,2048);
    }
};
/**
 *TransferSRMFileStat Structure (Public)
 */
struct TransferSRMStat {
    int  status;
    char message[2048];
    /**
     * Constructor
     */
    TransferSRMStat():status(-1){
        memset(message,0,2048);
    }
};

/*
 * Forwad declaration
 */
class Mmap_Status;
    
/**
 * Args for SRMCopy 
 */	
class TransferArgsSRM : public TransferArgsBase {
public:

	/**
     * Constructor
     */
    TransferArgsSRM(TransferList *l) : TransferArgsBase(),
		m_list(l),
        m_srmcopy_timeout(120),   // this is only the timeout on the soap srmcp call
        m_endpoint_dest(true){}

	/**
     * Get Transfer List
     */
    const TransferList *  list() const {return m_list;}
	/**
     * Get srmcopy timeout
     */
    int  srmcopy_timeout() const {return m_srmcopy_timeout;}
	/**
     * Set srmcopy timeout
     */
    void setSRMCopyTimeout (int srmcopy_timeout){m_srmcopy_timeout = srmcopy_timeout;}
	/**
     * Get Pull/push flag
     */
    bool is_dest_endpoint() const {return m_endpoint_dest;}
	/**
     * Set Pull/push flag
     */
    void useDestEndpoint(bool v) {m_endpoint_dest = v;}
private:
    TransferArgsSRM() {}
    TransferList *        m_list;
    int                   m_srmcopy_timeout;
    bool                  m_endpoint_dest;
};		

/**
 * The TransferCommands is the main class to interact with the transfer.
 */
class TransferSRM : public TransferBase {
public:

    typedef enum {
		FILE_STATUS_UNDEF   = -1,
		FILE_STATUS_PENDING = 0,
		FILE_STATUS_READY   = 1,
		FILE_STATUS_RUNNING = 2,
		FILE_STATUS_FAILED  = 3,
		FILE_STATUS_DONE    = 4,
	} FileStatus;

	typedef enum {	
		REQ_STATUS_UNDEF    = -1,
		REQ_STATUS_PENDING  = 5,
		REQ_STATUS_READY    = 6,
		REQ_STATUS_ACTIVE   = 7,
		REQ_STATUS_FAILED   = 8,
		REQ_STATUS_DONE     = 9
    } RequestStatus;

    /**
     * Constructor: Client Transfer object based on an existing ID
     */
    TransferSRM(const std::string &id, int http_timeout = 40 );

    /**
     * Destructor
     */
    virtual ~TransferSRM();

    // -------------------------------------------------------------------------
    // Transfer Methods
    // -------------------------------------------------------------------------
    
    /**
     * Spawn a transfer, retrieving the ID. This will not construct an
     * object but fork a separate process unless block is set to true,
     * in which case it blocks until the transfer is done.
     */
    static std::string start(TransferArgsSRM  & args, 
			                 std::vector<int> * fileId,
			                 int (*change_procname)(const char *) = NULL);

    /**
     * retrieve transfer status
     */
    TransferSocket::Status status(std::vector<TransferSRMFileStat> * filestats, 
                                  TransferSRMStat                  * transferstat,
                                  TransferLogger::Level loglevel = TransferLogger::INFO);

    /**
     * Abort transfer
     */
    void stop(TransferLogger::Level loglevel = TransferLogger::INFO);

    /**
     * Forget about this transfer (if SRM cannot be reached repeatedly for example)
     */
    void drop(TransferLogger::Level loglevel = TransferLogger::INFO);

    // -------------------------------------------------------------------------
    // Helper Method For the CLI
    // -------------------------------------------------------------------------
    static void dumpStatus(const std::string &file_name);
    
private:

    // -------------------------------------------------------------------------
    // Mmap status Methods
    // -------------------------------------------------------------------------
    
    /**
     * Create the Memory Mapped where the request stats will be stored
     */
    static void prepareMmap(const TransferArgsSRM & args, 
                                  TransferFiles   & transferfile,
                            const std::string     & endpoint) /*throw () */;

    /**
     * Read from the Memory Mapped file the file identifiers
     */
    static void readFileIdsFromMmap(TransferFiles    & transferfile,
                                    std::vector<int> & ids) /*throw () */;
     
    /**
     * update the Memory structure that contains the srmcopy stats
     */
    static void updateMmap(const class ::srm::srm2__RequestStatus * srm_stat, 
                                 class Mmap_Status                * stats) /*throw (TransferException) */;

    // -----------------------------------------------------------------------------
    // Internal functions (executed by the forked child process)
    // -----------------------------------------------------------------------------

    /**
     * Send an SRMCopy Request
     */
    static int callSRMCopy(const TransferArgsSRM & args,
                                 TransferFiles   & transfile,
                           const std::string     & endpoint);

    /**
     * Cancel an Active transfer
     */
    static int cancelActiveTransfer(class Mmap_Status    * stat, 
                                          TransferLogger & logger, 
                                    class srm::SrmUtil   & client) /*throw()*/;
    /**
     * Cleanup log files
     */
    static void cleanup(TransferFiles & transfile, 
                        TransferLogger& logger, 
                        const char    * msg,
                        bool            fail) /*throw()*/;

    // -----------------------------------------------------------------------------
    // translate states helpers functions
    // -----------------------------------------------------------------------------

    /**
     * Translate a string into a the Request state
     */
    static int getRequestStatus(char *state) /*throw()*/;
    /**
     * Translate a string into a the File state
     */
    static int getFileStatus(char *filestate) /*throw()*/;

private:
    /**
     * Default Constructor disabled
     */
    TransferSRM();

private:

	/**
	 * The Http timeout
	 */
	int m_http_timeout;
};

} // namespace transfer
} // namespace glite

#endif //GLITE_DATA_TRANSFER_SRM_HXX_
